@@ -1,6 +1,7 @@
+#!/bin/bash
 function hashencode(){
-        local str_map="$1"
-         echo `echo $str_map | sed 's/\./xDOTx/g' | sed 's/:/xCOLONx/g' | sed 's/,/:xCOMMAx/g'`
+    local str_map="$1"
+    echo `echo $str_map | sed 's/\./xDOTx/g' | sed 's/:/xCOLONx/g' | sed 's/,/:xCOMMAx/g' | sed 's/-/xHYPHENx/g'`
 }
 
 function hashset(){
@@ -107,7 +108,7 @@ function v4prefix2mask(){
     local num_index=1
     local str_temp=''
     local str_mask=''
-
+    [ ! $a ] && a=0
     while [[ $num_index -le 4 ]]
     do
         if [ $a -ge 8 ];then
@@ -208,7 +209,7 @@ function v6expand(){
         str_temp=$str_temp":0"
         num_index=$((num_index+1))
     done
-    
+
     str_v6address=`echo $str_v6address | sed "s/::/:$str_temp:/" | sed 's/^:/0:/' | sed 's/:$/:0/'`
     echo "$str_v6address"
 }
@@ -276,13 +277,13 @@ function v6calcnet(){
 function servicemap {
    local svcname=$1
    local svcmgrtype=$2
-   local svclistname=   
+   local svclistname=
 
-  # if there are more than 1 possible service names for a service among 
-  # different os distributions and os releases, the service should be 
-  # specified with structure 
-  # INIT_(general service name) = "list of possible service names"
-  #  
+   # if there are more than 1 possible service names for a service among
+   # different os distributions and os releases, the service should be
+   # specified with structure
+   # INIT_(general service name) = "list of possible service names"
+   #
    INIT_dhcp="dhcp3-server dhcpd isc-dhcp-server";
 
    INIT_nfs="nfsserver nfs-server nfs nfs-kernel-server";
@@ -290,7 +291,7 @@ function servicemap {
    INIT_named="named bind9";
 
    INIT_syslog="syslog syslogd rsyslog";
- 
+
    INIT_firewall="iptables firewalld ufw";
 
    INIT_http="apache2 httpd";
@@ -303,21 +304,39 @@ function servicemap {
 
    local path=
    local postfix=""
+   local svcmgrcmd=
    local retdefault=$svcname
    local svcvar=${svcname//[-.]/_}
    if [ "$svcmgrtype" = "0"  ];then
+      #for sysvinit
       path="/etc/init.d/"
+      svcmgrcmd="service"
    elif [ "$svcmgrtype" = "1"  ];then
+      #for systemd
       #retdefault=$svcname.service
-      path="/usr/lib/systemd/system/"
+      #ubuntu 16.04 replace upstart with systemd,
+      #all the service unit files are placed under /lib/systemd/system/ on ubuntu
+      #all the service unit files are placed under /usr/lib/systemd/system/ on redhat and sles
+      #path should be delimited with space
+      path="/usr/lib/systemd/system/ /lib/systemd/system/ /etc/systemd/system/"
       postfix=".service"
+      svcmgrcmd="systemctl"
    elif [ "$svcmgrtype" = "2"  ];then
+      #for upstart
       path="/etc/init/"
       postfix=".conf"
+      svcmgrcmd="initctl"
    fi
-   
+
+   #check whether the service management command exists
+   type $svcmgrcmd >/dev/null 2>&1
+   if [ $? -ne 0 ];then
+       echo ""
+       return
+   fi
+
    svclistname=INIT_$svcvar
-   local svclist=$(eval echo \$$svclistname)      
+   local svclist=$(eval echo \$$svclistname)
 
    if [ -z "$svclist" ];then
       svclist="$retdefault"
@@ -325,17 +344,20 @@ function servicemap {
 
    for name in `echo $svclist`
    do
-      if [ -e "$path$name$postfix"  ];then
-         echo $name
-         break
-      fi
+       for ipath in `echo $path`;do
+           if [ -e "$ipath$name$postfix"  ];then
+              echo "$name"
+              return
+           fi
+       done
    done
-   
+
+   echo ""
 }
 
 #some special services cannot be processed in sysVinit, upstart and systemd framework, should be process here...
 #Notice:
-# return value: 
+# return value:
 # 127 :         if the service $svcname cannot be processed in this function
 # otherwise:    the return value of the service action
 
@@ -364,7 +386,7 @@ function specialservicemgr {
               service SuSEfirewall2_setup status
               ;;
               *)
-              return 127 
+              return 127
               ;;
          esac
          return $?
@@ -378,7 +400,7 @@ function startservice {
    local svcname=$1
 
    local retval
-   specialservicemgr "$svcname" start 
+   specialservicemgr "$svcname" start
    retval=$?
    if [ "$retval" != "127"  ]; then
       return $retval
@@ -389,7 +411,7 @@ function startservice {
    local svcunit=`servicemap $svcname 1`
    local svcjob=`servicemap $svcname 2`
    local svcd=`servicemap $svcname 0`
-  
+
    if [ -n "$svcunit"  ];then
       cmd="systemctl start $svcunit"
    elif [ -n "$svcjob"  ];then
@@ -401,8 +423,16 @@ function startservice {
    if [ -z "$cmd"  ];then
       return 127
    fi
-   
-   eval $cmd
+
+   #for the linux distributions with systemd support
+   #In the chrooted env, the system management commands(start/stop/restart) will be ignored and the return code is 0
+   #need to return the proper code in the chrooted scenario
+   local retmsg
+   retmsg=`$cmd 2>&1`
+   retval=$?
+   [ "$retval" = "0" ] && (echo "$retmsg" | grep -i "Running in chroot,\s*ignoring request.*" >/dev/null 2>&1)   && retval=1
+
+   return $retval
 }
 
 
@@ -411,7 +441,7 @@ function stopservice {
    local svcname=$1
 
    local retval
-   specialservicemgr "$svcname" stop 
+   specialservicemgr "$svcname" stop
    retval=$?
    if [ "$retval" != "127"  ]; then
       return $retval
@@ -421,7 +451,7 @@ function stopservice {
    local svcunit=`servicemap $svcname 1`
    local svcjob=`servicemap $svcname 2`
    local svcd=`servicemap $svcname 0`
-  
+
    if [ -n "$svcunit"  ];then
       cmd="systemctl stop $svcunit"
    elif [ -n "$svcjob"  ];then
@@ -429,19 +459,23 @@ function stopservice {
       if [ "$?" = "0" ] ; then
          return 0
       else
-         cmd="initctl stop $svcjob"       
+         cmd="initctl stop $svcjob"
       fi
    elif [ -n "$svcd"  ];then
       cmd="service $svcd stop"
    fi
-    
-   echo $cmd
-
    if [ -z "$cmd"  ];then
       return 127
    fi
-   
-   eval $cmd
+   echo $cmd
+   #for the linux distributions with systemd support
+   #In the chrooted env, the system management commands(start/stop/restart) will be ignored and the return code is 0
+   #need to return the proper code in the chrooted scenario
+   local retmsg
+   retmsg=`$cmd 2>&1`
+   retval=$?
+   [ "$retval" = "0" ] && (echo "$retmsg" | grep -i "Running in chroot,\s*ignoring request.*" >/dev/null 2>&1) && retval=1
+   return $retval
 }
 
 
@@ -449,7 +483,7 @@ function restartservice {
    local svcname=$1
 
    local retval
-   specialservicemgr "$svcname" restart 
+   specialservicemgr "$svcname" restart
    retval=$?
    if [ "$retval" != "127"  ]; then
       return $retval
@@ -460,16 +494,16 @@ function restartservice {
    local svcunit=`servicemap $svcname 1`
    local svcjob=`servicemap $svcname 2`
    local svcd=`servicemap $svcname 0`
-  
+
    if [ -n "$svcunit"  ];then
       cmd="systemctl restart $svcunit"
    elif [ -n "$svcjob"  ];then
       initctl status $svcjob | grep stop
-      if [ "$?" = "0" ];then 
-         cmd= "initctl start $svcjob"
+      if [ "$?" = "0" ];then
+         cmd="initctl start $svcjob"
       else
          cmd="initctl restart $svcjob"
-      fi 
+      fi
    elif [ -n "$svcd"  ];then
       cmd="service $svcd restart"
    fi
@@ -477,8 +511,20 @@ function restartservice {
    if [ -z "$cmd"  ];then
       return 127
    fi
+
+   #for the linux distributions with systemd support
+   local retmsg
+   retmsg=`$cmd 2>&1`
+   retval=$?
    
-   eval $cmd
+   #If non 0 return code, display failure message
+   [ "$retval" = "1" ] && echo "$retmsg"
+
+   #In the chrooted env, the system management commands(start/stop/restart) will be ignored and the return code is 0
+   #need to return the proper code in the chrooted scenario
+   [ "$retval" = "0" ] && (echo "$retmsg" | grep -i "Running in chroot,\s*ignoring request.*" >/dev/null 2>&1) && retval=1
+
+   return $retval
 }
 
 
@@ -486,7 +532,7 @@ function checkservicestatus {
    local svcname=$1
 
    local retval
-   specialservicemgr "$svcname" status 
+   specialservicemgr "$svcname" status
    retval=$?
    if [ "$retval" != "127"  ]; then
       return $retval
@@ -496,9 +542,9 @@ function checkservicestatus {
    local svcunit=`servicemap $svcname 1`
    local svcjob=`servicemap $svcname 2`
    local svcd=`servicemap $svcname 0`
- 
-   local output= 
-   local retcode=3  
+
+   local output=
+   local retcode=3
 
    if [ -n "$svcunit"  ];then
       output=$(systemctl show --property=ActiveState $svcunit|awk -F '=' '{print $2}')
@@ -508,18 +554,20 @@ function checkservicestatus {
          retcode=1
       elif echo $output|grep -E -i "^failed$";then
          retcode=2
+      elif echo $output|grep -E -i "^activating$";then
+         retcode=17
       fi
    elif [ -n "$svcjob"  ];then
       output=$(initctl status $svcjob)
       if echo $output|grep -i "waiting";then
          retcode=2
-      elif echo $output|grep -i "running";then 
+      elif echo $output|grep -i "running";then
          retcode=0
-      fi 
+      fi
    elif [ -n "$svcd"  ];then
       output=$(service $svcd status)
       retcode=$?
-      echo $output 
+      echo $output
 #      if echo $output|grep -E -i "(stopped|not running)";then
 #         retcode=1
 #      elif echo $output|grep -E -i "running";then
@@ -528,16 +576,16 @@ function checkservicestatus {
    else
       retcode=127
    fi
-#echo $svcunit-----$svcd   
+#echo $svcunit-----$svcd
    return $retcode
-    
+
 }
 
 function enableservice {
    local svcname=$1
 
    local retval
-   specialservicemgr "$svcname" enable 
+   specialservicemgr "$svcname" enable
    retval=$?
    if [ "$retval" != "127"  ]; then
       return $retval
@@ -548,7 +596,7 @@ function enableservice {
    local svcunit=`servicemap $svcname 1`
    local svcjob=`servicemap $svcname 2`
    local svcd=`servicemap $svcname 0`
-  
+
    if [ -n "$svcunit"  ];then
       cmd="systemctl enable  $svcunit"
    elif [ -n "$svcjob"  ];then
@@ -561,14 +609,14 @@ function enableservice {
         command -v update-rc.d >/dev/null 2>&1
         if [ $? -eq 0 ];then
            cmd="update-rc.d $svcd defaults"
-        fi  
+        fi
      fi
    fi
 
    if [ -z "$cmd"  ];then
       return 127
    fi
-   
+
    eval $cmd
 }
 
@@ -577,7 +625,7 @@ function disableservice {
    local svcname=$1
 
    local retval
-   specialservicemgr "$svcname" disable 
+   specialservicemgr "$svcname" disable
    retval=$?
    if [ "$retval" != "127"  ]; then
       return $retval
@@ -587,7 +635,7 @@ function disableservice {
    local svcunit=`servicemap $svcname 1`
    local svcjob=`servicemap $svcname 2`
    local svcd=`servicemap $svcname 0`
-  
+
    if [ -n "$svcunit"  ];then
       cmd="systemctl disable  $svcunit"
    elif [ -n "$svcjob"  ];then
@@ -600,14 +648,14 @@ function disableservice {
         command -v update-rc.d >/dev/null 2>&1
         if [ $? -eq 0 ];then
            cmd="update-rc.d -f $svcd remove"
-        fi  
+        fi
      fi
    fi
 
    if [ -z "$cmd"  ];then
       return 127
    fi
-   
+
    eval $cmd
 }
 
@@ -615,10 +663,10 @@ declare -a array_nic_params
 declare -a array_extra_param_names
 declare -a array_extra_param_values
 
-# This function parse the NICEXTRAPARAMS into an array. 
+# This function parse the NICEXTRAPARAMS into an array.
 # Each arry element contains all the extra params for an ip
 # For example:
-#    
+#
 #    NICEXTRAPARAMS="eth0!MTU=1500 sonething=x|MTU=1460,ib0!MTU=65520 CONNECTED_MODE=yes"
 #    get_nic_extra_params $eth0 $NICEXTRAPARAMS
 #  After the function is called:
@@ -667,15 +715,16 @@ function get_nic_extra_params() {
 # This functions parse the extra parameters for an ip address of a nic
 # Input is like this:
 #     MTU=65520 something=yes
+#     MTU=65520 something=yes;no;maybe
 # After the function is called:
 #     array_extra_param_names[0]="MTU"
-#     array_extra_param_values[0]="65520" 
+#     array_extra_param_values[0]="65520"
 #     array_extra_param_names[1]="something"
-#     array_extra_param_values[0]="yes" 
-#     
+#     array_extra_param_values[1]="yes"  or array_extra_param_values[1]="yes no maybe"
+#
 function parse_nic_extra_params() {
     str_extra=$1
-   
+
     unset array_extra_param_names
     unset array_extra_param_values
 
@@ -688,7 +737,7 @@ function parse_nic_extra_params() {
 	do
 		token2="${params_temp[$k]}"
 		array_extra_param_names[$k]=`echo "$token2" | cut -d'=' -f 1`
-		array_extra_param_values[$k]=`echo "$token2" | cut -d'=' -f 2`	
+                array_extra_param_values[$k]=`echo "$token2" | cut -d'=' -f 2- | sed 's,\;, ,g'`
 		k=$((k+1))
 	done
 }
@@ -696,10 +745,10 @@ function parse_nic_extra_params() {
 #####################################################################
 #msgutil_r    : process messages from xcat postscripts
 #description  : for a specified message, call this function to echo,
-#               append to specified log file and log it remotely       
+#               append to specified log file and log it remotely
 #args         :
-#               logserver: the hostname or ip addr of log server               
-#               msgtype:   the type of the message, 
+#               logserver: the hostname or ip addr of log server
+#               msgtype:   the type of the message,
 #                          valid values(debug,info,warning,err)
 #               msgstr:    the string of the message  to process
 #               logfile:   the path of the file to append the log
@@ -709,25 +758,43 @@ function msgutil_r {
    local msgtype=$2
    local msgstr=$3
    local logfile=$4
+   local logtag=$5
 
-   if [ -z "$msgtype"  ]; then
+   if [ -z "$msgtype" ]; then
       msgtype="debug"
    fi
-
+   if [ -z "$logtag" ]; then
+      logtag="xcat"
+   fi
 
    if [ -n "$logserver" ];then
-      logger -n $logserver -t xcat -p local4.$msgtype "$msgstr" >/dev/null 2>&1
+       #In Ubuntu, there is a bug in some logger version that "-n" is ignored. The workaround is to specify the
+       #"-u <some dummy file>"  option together with "-n"  option.
+       #So far as we know, this bug exists in logger 2.20 shipped in ubuntu 14.04.4 and
+       #has been fixed in logger 2.25 shipped in ubuntu 15.04. Since "-u" option won't break anything,
+       #use "-u xxxx" for the logger versions under 2.25
+        if [ -f /etc/os-release ] && cat /etc/os-release |grep -i NAME|grep -i Ubuntu>/dev/null 2>&1
+        then
+            eval $(logger -V 2>/dev/null | awk '{print $4}' | awk -F '.' '{printf("var1=%s; var2=%s",$1,$2)}')
+            if [ $var1 -eq 2 ] && [ $var2 -lt 25 ]; then
+                logger -u /tmp/ignored -n $logserver -t $logtag -p local4.$msgtype "$msgstr" >/dev/null 2>&1
+            else
+                logger -n $logserver -t $logtag -p local4.$msgtype "$msgstr" >/dev/null 2>&1
+            fi
+        else
+            logger -n $logserver -t $logtag -p local4.$msgtype "$msgstr" >/dev/null 2>&1
+        fi
       if [ "$?" != "0" ];then
-         exec 3<>/dev/udp/$logserver/514 >/dev/null 2>&1;logger -s -t xcat -p local4.$msgtype "$msgstr" 1>&3  2>&1
+         exec 3<>/dev/udp/$logserver/514 && logger -s -t $logtag -p local4.$msgtype "$msgstr" 1>&3  2>&1 && exec 3>&-
          if [ "$?" != "0" ];then
-            logger -s -t xcat -p local4.$msgtype "$msgstr" 2>&1|nc $logserver 514 >/dev/null 2>&1
+            logger -s -t $logtag -p local4.$msgtype "$msgstr" 2>&1|nc $logserver 514 >/dev/null 2>&1
             if [ "$?" != "0" ];then
-               logger -t xcat -p local4.$msgtype "$msgstr"
+               logger -t $logtag -p local4.$msgtype "$msgstr"
             fi
          fi
       fi
    else
-       logger -t xcat -p local4.$msgtype "$msgstr"
+       logger -t $logtag -p local4.$msgtype "$msgstr"
    fi
 
    if [ -n "$logfile"  ]; then
@@ -739,7 +806,7 @@ function msgutil_r {
 
       #   echo "$msgstr" | tee -a $logfile
       #else
-      echo "$msgstr" >> $logfile
+      echo "$(date) [$msgtype]: $logtag: $msgstr" >> $logfile
    fi
 
 }
@@ -748,14 +815,15 @@ function msgutil_r {
 
 #####################################################################
 #msgutil      : process messages from xcat postscripts
-#description  : for a specified message, call this function to 
-#               echo,append to the specified file and log it        
-#args         : 
-#               msgtype:  the type of the message, 
+#description  : for a specified message, call this function to
+#               echo,append to the specified file and log it
+#args         :
+#               msgtype:  the type of the message,
 #                         valid values(debug,info,warning,err)
 #               msgstr:   the string of the message  to process
-#               logfile:  the path of the file to append the log 
+#               logfile:  the path of the file to append the log
 ####################################################################
 function msgutil {
    msgutil_r "" "$@"
 }
+
